/*
Abacus
Copyright (C) 2025  Jeffry Johnston

This file is part of Abacus.

Abacus is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Abacus is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Abacus.  If not, see <https://www.gnu.org/licenses/>.
*/

#include <gint/display.h>
#include <gint/gint.h>
#include <gint/hardware.h>
#include <gint/keyboard.h>
#ifndef FX9860G_G3A
#include <gint/drivers/t6k11.h>
#include <gint/usb-ff-bulk.h>
#endif
#include <stdbool.h>

#define COLUMNS 10

extern bopti_image_t img_active_half;
extern bopti_image_t img_background;
extern bopti_image_t img_bead;
extern bopti_image_t img_help_left;
extern bopti_image_t img_help_left_cg100;
extern bopti_image_t img_help_right;
extern bopti_image_t img_help_right_cg100;

static bool display_help;
static uint8_t left_column = 0;
static uint8_t beads_top[COLUMNS];
static uint8_t beads_bottom[COLUMNS];

static void reset_beads(void)
{
	for (int i = 0; i < COLUMNS; ++i) {
		beads_top[i] = 2;
		beads_bottom[i] = 0;
	}
}

static void init(void)
{
#ifndef FX9860G_G3A
	usb_interface_t const *interfaces[] = { &usb_ff_bulk, NULL };
	usb_open(interfaces, GINT_CALL_NULL);
#endif

	display_help = false;
	left_column = 5;
	reset_beads();
}

#ifndef FX9860G_G3A
/*
How to take an in-game screenshot:
1. Run: fxlink -iw
2. Connect the calculator via USB then press EXIT (don't press F1 - F3)
3. Press "log" in the game
*/
static void take_screenshot(void)
{
	if (usb_is_open())
		usb_fxlink_screenshot(1);
}
#endif

static void display_abacus(void)
{
	if (display_help) {
		if (gint[HWCALC] == HWCALC_FXCG100)
			dimage(0, 0, (left_column == 0) ? &img_help_left_cg100 : &img_help_right_cg100);
		else
			dimage(0, 0, (left_column == 0) ? &img_help_left : &img_help_right);
	} else {
		dimage(0, 0, &img_background);
		dimage((left_column == 0) ? 0 : 64, 22, &img_active_half);
		for (int i = 0; i < COLUMNS; ++i) {
			const uint8_t x = 13 * i;
			for (int row = 0; row < 3; ++row) {
				if (beads_top[i] != row)
					dimage(x, 6 * row + 3, &img_bead);
			}
			for (int row = 0; row < 6; ++row) {
				if (beads_bottom[i] != row)
					dimage(x, 6 * row + 25, &img_bead);
			}
		}
	}
	dupdate();
}

static unsigned int kbd_getkey(void)
{
	const key_event_t event = getkey_opt(GETKEY_BACKLIGHT | GETKEY_REP_ALL, NULL);
	const uint key = event.key;
	switch (key) {
#ifndef FX9860G_G3A
	case KEY_LOG:
		take_screenshot();
		break;
#endif

#ifndef FX9860G_G3A
	case KEY_OPTN:
		if (!isSlim())
			t6k11_backlight(-1);
		break;
#endif
	}
	return key;
}

int main(void)
{
	init();
	while (1) {
		display_abacus();

		switch (kbd_getkey()) {
		case KEY_MENU:
			if (gint[HWCALC] == HWCALC_FXCG100)
				return 0;
			gint_osmenu();
			break;
		case KEY_F1:
		case KEY_F2:
		case KEY_F3:
		case KEY_F4:
		case KEY_F5:
		case KEY_F6:
		case KEY_HELP:
		case KEY_CATALOG:
		case KEY_SETTINGS:
			display_help = !display_help;
			break;
		case KEY_EXIT:
			display_help = false;
			break;
		case KEY_LEFT:
		case KEY_RIGHT:
			if (!display_help)
				left_column = 5 - left_column;
			break;
		case KEY_SHIFT:
			if (!display_help)
				reset_beads();
			break;
		case KEY_7:
			if (!display_help) {
				if (beads_top[left_column + 0] < 2)
					++beads_top[left_column + 0];
			}
			break;
		case KEY_8:
			if (!display_help) {
				if (beads_top[left_column + 1] < 2)
					++beads_top[left_column + 1];
			}
			break;
		case KEY_9:
			if (!display_help) {
				if (beads_top[left_column + 2] < 2)
					++beads_top[left_column + 2];
			}
			break;
		case KEY_DEL:
			if (!display_help) {
				if (beads_top[left_column + 3] < 2)
					++beads_top[left_column + 3];
			}
			break;
		case KEY_ACON:
			if (!display_help) {
				if (beads_top[left_column + 4] < 2)
					++beads_top[left_column + 4];
			}
			break;
		case KEY_4:
			if (!display_help) {
				if (beads_top[left_column + 0])
					--beads_top[left_column + 0];
			}
			break;
		case KEY_5:
			if (!display_help) {
				if (beads_top[left_column + 1])
					--beads_top[left_column + 1];
			}
			break;
		case KEY_6:
			if (!display_help) {
				if (beads_top[left_column + 2])
					--beads_top[left_column + 2];
			}
			break;
		case KEY_MUL:
			if (!display_help) {
				if (beads_top[left_column + 3])
					--beads_top[left_column + 3];
			}
			break;
		case KEY_DIV:
			if (!display_help) {
				if (beads_top[left_column + 4])
					--beads_top[left_column + 4];
			}
			break;
		case KEY_1:
			if (!display_help) {
				if (beads_bottom[left_column + 0] < 5)
					++beads_bottom[left_column + 0];
			}
			break;
		case KEY_2:
			if (!display_help) {
				if (beads_bottom[left_column + 1] < 5)
					++beads_bottom[left_column + 1];
			}
			break;
		case KEY_3:
			if (!display_help) {
				if (beads_bottom[left_column + 2] < 5)
					++beads_bottom[left_column + 2];
			}
			break;
		case KEY_ADD:
			if (!display_help) {
				if (beads_bottom[left_column + 3] < 5)
					++beads_bottom[left_column + 3];
			}
			break;
		case KEY_SUB:
			if (!display_help) {
				if (beads_bottom[left_column + 4] < 5)
					++beads_bottom[left_column + 4];
			}
			break;
		case KEY_0:
			if (!display_help) {
				if (beads_bottom[left_column + 0])
					--beads_bottom[left_column + 0];
			}
			break;
		case KEY_DOT:
			if (!display_help) {
				if (beads_bottom[left_column + 1])
					--beads_bottom[left_column + 1];
			}
			break;
		case KEY_EXP:
			if (!display_help) {
				if (beads_bottom[left_column + 2])
					--beads_bottom[left_column + 2];
			}
			break;
		case KEY_NEG:
		case KEY_FORMAT:
			if (!display_help) {
				if (beads_bottom[left_column + 3])
					--beads_bottom[left_column + 3];
			}
			break;
		case KEY_EXE:
			if (!display_help) {
				if (beads_bottom[left_column + 4])
					--beads_bottom[left_column + 4];
			}
			break;
		}
	}
	return 1;
}
